{% load i18n %}
{% load humanize %}

{% if stats.source_strings %}
  <tr>
    <th>{% trans "Languages" context "Number of used languages" %}</th>
    <td class="full-cell number">{{ stats.languages|intcomma }}</td>
  </tr>

  <tr>
    <th>{% trans "Source strings" context "Number of source strings" %}</th>
    <td class="full-cell number">{{ stats.source_strings|intcomma }}</td>
  </tr>

  <tr>
    <th>{% trans "Source words" context "Number of source words" %}</th>
    <td class="full-cell number">{{ stats.source_words|intcomma }}</td>
  </tr>

  <tr>
    <th>{% trans "Source characters" context "Number of source characters" %}</th>
    <td class="full-cell number">{{ stats.source_chars|intcomma }}</td>
  </tr>
{% endif %}

<tr>
  <th>{% trans "Hosted strings" context "Number of hosted strings" %}</th>
  <td class="full-cell number">{{ stats.all|intcomma }}</td>
</tr>

<tr>
  <th>{% trans "Hosted words" context "Number of hosted words" %}</th>
  <td class="full-cell number">{{ stats.all_words|intcomma }}</td>
</tr>

<tr>
  <th>{% trans "Hosted characters" context "Number of hosted characters" %}</th>
  <td class="full-cell number">{{ stats.all_chars|intcomma }}</td>
</tr>
