{% load i18n %}
{% load icons %}
<div class="tab-pane" id="activity">

<h3>{% trans "Daily activity" %}</h3>
<div class="activity-graph">
    <img src="{{ monthly_activity }}" loading="lazy" alt="{% trans "Daily activity" %}" />
</div>

<h3>{% trans "Weekly activity" %}</h3>
<div class="activity-graph">
    <img src="{{ yearly_activity }}" loading="lazy" alt="{% trans "Weekly activity" %}" />
</div>

</div>
